@@ -29,10 +29,10 @@
 #include <string.h>
 #include <math.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <gtk/gtksignal.h>
 
 #include <libgnomecanvas/gnome-canvas-clipgroup.h>
 #include <libgnomeprint/private/gp-gc-private.h>
+#include <libgnomeprint/private/gnome-glyphlist-private.h>
 
 #include "gnome-canvas-hacktext.h"
 #include "gnome-print-preview-private.h"
@@ -49,10 +49,26 @@ struct _GnomePrintPreviewPrivate {
 
 	/* The current page */
 	GnomeCanvasItem *page;
+
+	/* Strict theme compliance [#96802] */
+	gboolean theme_compliance; 
 };
 
 static GnomePrintContextClass *parent_class;
 
+static void
+outline_set_style_cb (GtkWidget *canvas, GtkStyle *ps, GnomeCanvasItem *item)
+{
+	gint32 color;
+	GtkStyle *style;
+
+	style = gtk_widget_get_style (GTK_WIDGET (canvas));
+	color = GPP_COLOR_RGBA (style->text [GTK_STATE_NORMAL], 0xff);
+	
+	gnome_canvas_item_set (item, "outline_color_rgba", color, NULL);
+}
+	
+
 static int
 gpp_stroke (GnomePrintContext *pc, const ArtBpath *bpath)
 {
@@ -86,9 +102,26 @@ gpp_stroke (GnomePrintContext *pc, const ArtBpath *bpath)
 
 	gnome_canvas_path_def_unref (path);
 
+	if (pp->priv->theme_compliance) {
+		g_signal_connect (G_OBJECT (item->canvas), "style_set",
+				  G_CALLBACK (outline_set_style_cb), item);
+	}
+	
 	return 1;
 }
 
+static void
+fill_set_style_cb (GtkWidget *canvas, GtkStyle *ps, GnomeCanvasItem *item)
+{
+	gint32 color;
+	GtkStyle *style;
+
+	style = gtk_widget_get_style (GTK_WIDGET (canvas));
+	color = GPP_COLOR_RGBA (style->text [GTK_STATE_NORMAL], 0xff);
+	
+	gnome_canvas_item_set (item, "fill_color_rgba", color, NULL);
+}
+	
 static int
 gpp_fill (GnomePrintContext *pc, const ArtBpath *bpath, ArtWindRule rule)
 {
@@ -114,9 +147,13 @@ gpp_fill (GnomePrintContext *pc, const ArtBpath *bpath, ArtWindRule rule)
 		"fill_color_rgba", gp_gc_get_rgba (pc->gc),
 		"wind", rule,
 		NULL);
-
 	gnome_canvas_path_def_unref (path);
 
+	if (pp->priv->theme_compliance) {
+		g_signal_connect (G_OBJECT (item->canvas), "style_set",
+				  G_CALLBACK (fill_set_style_cb), item);
+	}
+
 	return 1;
 }
 
@@ -246,13 +283,34 @@ gpp_image (GnomePrintContext *pc, const gdouble *affine, const guchar *px, gint
 static int
 gpp_showpage (GnomePrintContext *pc)
 {
-	return 0;
+	return GNOME_PRINT_OK;
 }
 
 static int
 gpp_beginpage (GnomePrintContext *pc, const guchar *name)
 {
-	return 0;
+	return GNOME_PRINT_OK;
+}
+
+static void
+glyphlist_set_style_cb (GtkWidget *canvas, GtkStyle *ps, GnomeCanvasItem *item)
+{
+	GnomeGlyphList *gl, *new;
+	gint32 color;
+	GtkStyle *style;
+	gint i;
+
+	style = gtk_widget_get_style (GTK_WIDGET (canvas));
+	color = GPP_COLOR_RGBA (style->text [GTK_STATE_NORMAL], 0xff);
+
+	g_object_get (G_OBJECT (item), "glyphlist", &gl, NULL);
+	new = gnome_glyphlist_duplicate (gl);
+	for (i = 0; i < new->r_length; i++) {
+		if (new->rules[i].code ==  GGL_COLOR) {
+			new->rules[i].value.ival = color;
+		}
+	}
+	gnome_canvas_item_set (item, "glyphlist", new, NULL);
 }
 
 static int
@@ -273,7 +331,6 @@ gpp_glyphlist (GnomePrintContext *pc, const gdouble *affine, GnomeGlyphList * gl
 	art_affine_multiply (transform, a, affine);
 
 	group = (GnomeCanvasGroup *) gp_gc_get_data (pp->priv->gc);
-
 	item = gnome_canvas_item_new (group,
 				      gnome_canvas_hacktext_get_type (),
 				      "x", 0.0,
@@ -283,6 +340,11 @@ gpp_glyphlist (GnomePrintContext *pc, const gdouble *affine, GnomeGlyphList * gl
 
 	gnome_canvas_item_affine_absolute (item, transform);
 
+	if (pp->priv->theme_compliance) {
+		g_signal_connect (G_OBJECT (item->canvas), "style_set",
+				  G_CALLBACK (glyphlist_set_style_cb), item);
+	}
+	
 	return 0;
 }
 
@@ -374,6 +436,21 @@ clear_val (GtkObject *object, void **val)
 	*val = NULL;
 }
 
+/**
+ * gnome_print_preview_theme_compliance:
+ * @pp: 
+ * @compliance: 
+ * 
+ * This has to go away for GNOME 2.4. Make compliance an argument of
+ * gnome_print_preview_new_full
+ **/
+void
+gnome_print_preview_theme_compliance (GnomePrintPreview *pp,
+				      gboolean compliance)
+{
+	pp->priv->theme_compliance = compliance;
+}
+
 GnomePrintContext *
 gnome_print_preview_new_full (GnomePrintConfig *config, GnomeCanvas *canvas,
 			      const gdouble *transform, const ArtDRect *region)
@@ -393,6 +470,7 @@ gnome_print_preview_new_full (GnomePrintConfig *config, GnomeCanvas *canvas,
 	ret = gnome_print_context_construct (ctx, config);
 	if (ret != GNOME_PRINT_OK) {
 		g_object_unref (ctx);
+		g_warning ("Could not construct the GnomePrintPreview context\n");
 		return NULL;
 	}
 	preview = GNOME_PRINT_PREVIEW (ctx);
